﻿using System;
using System.Threading.Tasks;
using Sandbox.Interfaces.Repositories.Abstract;
using Sandbox.Databases;
using Microsoft.EntityFrameworkCore;
using System.Reflection;
using System.Linq;
using Sandbox.Models.Abstract;

namespace Sandbox.Repositories.Abstract
{
    public abstract class EntityRepository<T> : IEntityRepository<T> where T: Entity
    {
        protected readonly ModelsContext context;
        protected readonly DbSet<T> persistentCollection;

        public EntityRepository(ModelsContext context)
        {
            this.context = context;

            var property = typeof(ModelsContext).GetProperties().SingleOrDefault(x => x.PropertyType.Equals(typeof(DbSet<T>)));
            if (property == null || !property.CanRead)
            {
                throw new ArgumentException("DbSet for model " + typeof(T).Name + " does not exist in the current context.");
            }

            this.persistentCollection = (DbSet<T>)property.GetValue(context, null);
        }

        public virtual async Task<T> Create(T instance)
        {
            var entityEntry = this.persistentCollection.Add(instance);
            await this.context.SaveChangesAsync();

            return entityEntry.Entity;
        }

        public virtual async void Delete(T instance)
        {
            var entityEntry = this.persistentCollection.Remove(instance);
            await this.context.SaveChangesAsync();
        }

        public virtual async Task<T> Retrieve(int id)
        {
            var entity = await this.persistentCollection.SingleOrDefaultAsync(x => x.Id.Equals(id));
            return entity;
        }

        public virtual Task<IQueryable<T>> RetrieveAll()
        {
            return Task.FromResult(this.persistentCollection.AsQueryable());
        }

        public virtual async Task<T> Update(T instance)
        {
            var entityEntry = this.persistentCollection.Update(instance);
            await this.context.SaveChangesAsync();

            return entityEntry.Entity;
        }
    }
}